<?php
/*
Plugin Name: GravatarLocalCache
Website link: http://blog.splash.de/
Author URI: http://blog.splash.de/
Plugin URI: http://blog.splash.de/2010/01/04/gravatarlocalcache
Description: Local cache for gravatar images (saves dnsqueries and let you control cache/proxysettings of the images).
Author: Oliver Schaal
Version: 1.1.2
*/

if (!function_exists('is_admin')) {
header('Status: 403 Forbidden');
header('HTTP/1.1 403 Forbidden');
exit();
}
require_once(ABSPATH . 'wp-admin/includes/file.php');
$stylesheet=get_option('stylesheet');
define('GLCCACHEPATH', get_stylesheet_directory().'/gravatars/');
define('GLCCACHEURL', get_theme_root_uri() . '/' . $stylesheet . '/gravatars/' );

if (!class_exists("GravatarLocalCache")) {
class GravatarLocalCache {

private static $defaultAvatar = 'mystery';
private static $count = 0;

public function __construct() {
self::$defaultAvatar = get_option('avatar_default');

if (is_user_logged_in()) {
add_action('wp_dashboard_setup', array(&$this, 'initAdminDashboard'));
}
if ( is_dir(GLCCACHEPATH) || ( umask(0022) && @mkdir( GLCCACHEPATH , 0755, true ) ) ) {
add_filter('get_avatar', array(&$this, 'getGravatarLocalCacheFilter'), 1, 2);
}
}

public function initAdminDashboard() {
wp_add_dashboard_widget( 'GravatarLocalCacheWidget', __( 'Stallion Gravatar Cache Stats', 'stallion' ), array(&$this, 'showAdminDashboard') );
}

public function showAdminDashboard() {
echo '<div class="inside">
<ul>';
if (is_dir(GLCCACHEPATH)) {
self::cleanupCache(GLCCACHEPATH);
$_filesInCache = self::filesInDirectory(GLCCACHEPATH);
$_fileInCacheCount = count($_filesInCache);
if ($_fileInCacheCount > 0) {

echo '<li><strong>'
 .__('Images in cache', 'stallion').':</strong> '
 .$_fileInCacheCount.'</li>';

// total size
foreach ($_filesInCache as $_key => $_row) {
$_size[$_key]  = $_row['size'];
}
$_totalSize = 0;
foreach($_size as $_key => $_value) {
$_totalSize += $_value;
}
echo '<li><strong>'.__('Total size', 'stallion').':</strong> '.self::fileSizeHumanReadable($_totalSize).'</li>';

// size
$_filesInCache = self::array_msort($_filesInCache, array('size'=>SORT_ASC, 'name'=>SORT_ASC));
reset($_filesInCache);
$_first=current($_filesInCache);
$_last=end($_filesInCache);
echo '<li><strong>'.__('Size (smallest/largest)', 'stallion').':</strong></li><li>'
.self::fileSizeHumanReadable($_first['size']).' / '
.self::fileSizeHumanReadable($_last['size']).'</li>';

// date
$_filesInCache = self::array_msort($_filesInCache, array('mdate'=>SORT_ASC, 'name'=>SORT_ASC));
reset($_filesInCache);
$_first=current($_filesInCache);
$_last=end($_filesInCache);
echo '<li><strong>'.__('Date (youngest/oldest)', 'stallion').':</strong></li><li>'
.date ("d-m-Y H:i:s", $_last['mdate']).' / '
.date ("d-m-Y H:i:s", $_first['mdate']).'</li>';
} else {
_e('There are no cached images.', 'stallion');
}
}
echo '</ul>
</div>';
}

private static function callbackGravatarLocalCache($matches) {
$_imageurl = $matches[1];
preg_match('|http://[0-9a-z]{1,3}.gravatar.com/avatar/([^\?]+)\?s=([0-9]+)|i', $_imageurl, $_img);
$_name = $_img[1];
$_file = GLCCACHEPATH.$_name.'-'.$_img[2].'.jpg';
$_url = GLCCACHEURL.$_name.'-'.$_img[2].'.jpg';

if (file_exists($_file) && ((time() - filemtime($_file))/60/60) < st_gravatar_cache_time()) {
$_retVal = $_url;
} else {

if (!empty($_imageurl) && self::$count < 20 ) {
self::$count++;

$_tmp = download_url(str_replace(utf8_encode('&amp;'), '&', $_imageurl));

if ( is_wp_error($_tmp) ) {
@unlink($_tmp);

preg_match('/&amp;d=(.*)/i',$_imageurl,$_defaultavatar);
$_tmp = download_url(urldecode($_defaultavatar[1]));
if ( is_wp_error($_tmp) ) {
@unlink($_tmp);

if ( self::$defaultAvatar == 'mystery') {
$_defavatar = 'http://www.gravatar.com/avatar/'.md5('unknown@gravatar.com').'?s='.$_img[2];
} elseif (self::$defaultAvatar == 'blank') {
$_defavatar = includes_url('images/blank.gif');
} else {
$_defavatar = 'http://www.gravatar.com/avatar/?d='.self::$defaultAvatar.'&s='.$_img[2];
}

$_tmp = download_url($_defavatar);
if ( is_wp_error($_tmp) ) {
@unlink($_tmp);
return $matches[0];
}
}
}

if ( false === @rename( $_tmp, $_file ) ) {
return $matches[0];
} else {
$_stat = stat( dirname( $_file ));
$_perms = $_stat['mode'] & 0000666;
@chmod( $_file, $_perms );

$_retVal = $_url;
}
##########################################@unlink($_tmp);

} elseif (file_exists($_file)) {
$_retVal = $_url;
} else {
return $matches[0];
}
}

return str_replace($_imageurl,$_retVal,$matches[0]);
}

public function getGravatarLocalCacheFilter($content, $arg) {
return preg_replace_callback('|<img.*src=\'([^\']+)\'.*/>|i', array( &$this, 'callbackGravatarLocalCache'), $content);
}

private static function cleanupCache ($path) {
if ($handle = opendir($path)) {
while (false !== ($file = readdir($handle))) {
if ($file[0] == '.' || is_dir($path.'/'.$file)) {
continue;
}

if ((time() - filemtime($path.'/'.$file)) > (st_gravatar_cache_time() *60*60)) {
@unlink($path.'/'.$file);
#unlink($path.'/'.$file);
}
}
closedir($handle);
}
}

function array_msort($array, $cols) {
$colarr = array();
foreach ($cols as $col => $order) {
$colarr[$col] = array();
foreach ($array as $k => $row) {
$colarr[$col]['_'.$k] = strtolower($row[$col]);
}
}
$params = array();
foreach ($cols as $col => $order) {

$params[] =&$colarr[$col];
$order=(array)$order;
foreach($order as $order_element) {
$params[]=&$order_element;
}
}
call_user_func_array('array_multisort', $params);
$ret = array();
$keys = array();
$first = true;
foreach ($colarr as $col => $arr) {
foreach ($arr as $k => $v) {
if ($first) {
$keys[$k] = substr($k,1);
}
$k = $keys[$k];

if (!isset($ret[$k])) {
$ret[$k] = $array[$k];
}

$ret[$k][$col] = $array[$k][$col];
}
$first = false;
}
return $ret;
}

private static function fileSizeHumanReadable($bytes, $precision = 2) {
$units = array('B', 'KB', 'MB', 'GB', 'TB');

$bytes = max($bytes, 0);
$pow = floor(($bytes ? log($bytes) : 0) / log(1024));
$pow = min($pow, count($units) - 1);

$bytes /= pow(1024, $pow);

return round($bytes, $precision) . ' ' . $units[$pow];
}

private static function filesInDirectory($path, $exclude = ".|..", $recursive = false) {
$path = rtrim($path, "/") . "/";
$folder_handle = opendir($path);
$exclude_array = explode("|", $exclude);
$result = array();
$i=0;
while(false !== ($filename = readdir($folder_handle))) {
if(!in_array(strtolower($filename), $exclude_array)) {
if(is_dir($path . $filename)) {
if($recursive) $result[] = file_array($path, $exclude, true);
} else {
$i++;
$result[$i]['name'] = $filename;
$result[$i]['size'] = filesize($path.$filename);
$result[$i]['mdate'] = filemtime($path.$filename);
}
}
}
return $result;
}

}
}

if (class_exists("GravatarLocalCache")) {
$gravatarLocalCache = new GravatarLocalCache();
} ?>