<?php
/*
Plugin Name:WP Widget Cache
Plugin URI: https://github.com/rooseve/wp-widget-cache
Description: Cache the output of your blog widgets. Usually it will significantly reduce the sql queries to your database and speed up your site.
Author: Andrew Zhang
Version: 0.26
Author URI: https://github.com/rooseve/wp-widget-cache
*/
if (!function_exists('is_admin')) {
header('Status: 403 Forbidden');
header('HTTP/1.1 403 Forbidden');
exit();
}

class WCache
{
var $dir_mode = 0755;
var $surephp5 = false;
var $fld_version = '__wgc_v';
var $fld_output = 'output';
var $fld_data = 'data';
/**
*
* @param string $path,
*        the cache dir path
* @param string $disable
*        disable the cache or not
* @param string $disable_output
*        disable track the output(ob_start) or not
*/
function WCache($cache_dir, $disable = false, $disable_output = false)
{
if (function_exists ( "version_compare" ))
{
$this->surephp5 = version_compare ( PHP_VERSION, '5.0.0', '>=' );
}

//make cache dir
if (! is_dir ( $cache_dir ))
{
$this->__do_mkdir ( $cache_dir, $this->dir_mode );
$disable = ! is_dir ( $cache_dir );
}

//writable or not
if (! $disable && ! is_writable ( $cache_dir ))
{
@chmod ( $cache_dir, $this->dir_mode );
$disable = ! is_writable ( $cache_dir );
}

//make sure path ends with /
if (! in_array ( substr ( $cache_dir, - 1 ), array (
"\\",
"/" 
) ))
{
$cache_dir .= "/";
}

$this->path = $cache_dir;
$this->disable = $disable;
$this->disable_output = $disable_output;
$this->stack = array ();
$this->output = null;
}
function save($key, $expire_timespan, $cdata = null, $group = false)
{
if ($this->disable)
{
//nothing to do
return false;
}

$keypath = $this->__get_key_path ( $key, $group );

$expire_timespan = max ( 3, intval ( $expire_timespan ) );

//here the real data created
if (count ( $this->stack ) && $keypath == $this->stack [count ( $this->stack ) - 1])
{
$ob_output = false;

if (! $this->disable_output)
{
$ob_output = ob_get_contents ();
ob_end_clean ();

$this->__echo_output ( $ob_output );
}

//create a cache pack
$cpack = array ();
$cpack [$this->fld_version] = 2;
$cpack [$this->fld_output] = $ob_output;
$cpack [$this->fld_data] = $cdata;

$this->__save_cache ( $keypath, $cpack );

unset ( $this->stack [count ( $this->stack ) - 1] );

return false;
}
elseif (count ( $this->stack ) && in_array ( $keypath, $this->stack ))
{
trigger_error ( 
"Cache stack problem: " . $this->stack [count ( $this->stack ) - 1] . " not properly finished!", 
E_USER_ERROR );
return false;
}
else
{
$res = $this->__start_track ( $keypath, $expire_timespan );

//well no cache available
if (is_int ( $res ))
{
if (! $this->disable_output)
{
//track the output
ob_start ();
}

return $res;
}
else
{
//old version cache data
if (! isset ( $res [$this->fld_version] ))
{
$res [$this->fld_version] = 1;
}

$res_output = false;
$res_cdata = array ();

switch ($res [$this->fld_version])
{
case 1 :

if (isset ( $res ['__output__'] ))
{
$res_output = $res ['__output__'];
unset ( $res ['__output__'] );
}

$res_cdata = $res;

break;

default :

$res_output = $res [$this->fld_output];
$res_cdata = $res [$this->fld_data];

break;
}

$this->__echo_output ( $res_output );

if (is_array ( $cdata ))
{
//copy the cdata
foreach ( $res_cdata as $k => $v )
{
$cdata [$k] = $res_cdata [$v];
}
}

return false;
}
}
}

/**
 * Remove all caches
 *
 * @param number $expire_timespan        
 * @return number
 */
function clear($expire_timespan = 0)
{
return $this->__scan_dir ( $this->path, $expire_timespan );
}

/**
 * Remove a single cache
 *
 * @param string $key        
 * @param string $group        
 */
function remove($key, $group = false)
{
if (! $key)
{
return;
}

$keypath = $this->__get_key_path ( $key, $group );

$this->__remove_cache ( $keypath );
}

/**
 * Remove caches in a group
 *
 * @param string $group        
 */
function remove_group($group)
{
if (! $group)
{
return;
}

$subdir = $this->__encode_key ( $group );

$this->__remove_cache ( $subdir );
}

/**
 * How many caches in the cache dir
 *
 * @return number
 */
function cachecount() {
return $this->__scan_dir ( $this->path, - 100 );
}

function __echo_output($output) {
$this->output = $output;

if (! $this->disable_output) {
echo $output;
}
}

function __remove_cache($keypath) {
$filename = $this->path . $keypath;

if (is_file ( $filename )) {
@unlink ( $filename );
return true;
}
else if (is_dir ( $filename )) {
$this->__scan_dir ( $filename, 0 );
}

return false;
}

function __save_cache($keypath, $data) {
if ($this->disable) {
return false;
}

$filename = $this->path . $keypath;

if (file_exists ( $filename ) && ! is_writable ( $filename )) {
trigger_error ( "Cache file not writeable!", E_USER_ERROR );
return false;
}

$f = fopen ( $filename, 'w' );
if (flock ( $f, LOCK_EX )) {
fwrite ( $f, $this->__unpack_data ( $data ) );
flock ( $f, LOCK_UN );
}
fclose ( $f );

return true;
}

function __load_cache($keypath, $expire_timespan) {
if ($this->disable)
{
return false;
}

$filename = $this->path . $keypath;

if (! file_exists ( $filename )) {
return false;
}

#if (time () - filemtime ( $filename ) > $expire_timespan)
if (time () - filemtime ( $filename ) > ($expire_timespan *60*60*24)) {
return false;
}
return @file_get_contents ( $filename );
}

function __start_track($keypath, $time) {
$data = $this->__load_cache ( $keypath, $time );

//no cache available
if ($data === false) {
//push it to stack
$this->stack [count ( $this->stack )] = $keypath;

return count ( $this->stack );
}

$data = $this->__pack_data ( $data );

return $data;
}

function __pack_data($data) {
return unserialize ( $data );
}

function __unpack_data($data) {
return serialize ( $data );
}

function __encode_key($name) {
return md5 ( $name );
}

function __get_key_path($key, $group = false) {
if (! is_string ( $key )) {
$key = serialize ( $key );
}

$key = $this->__encode_key ( $key );

if ($group) {
if (! is_string ( $group ))
$group = serialize ( $group );

$subdir = $this->__encode_key ( $group );

if (! is_dir ( $this->path . $subdir )) {
$this->__do_mkdir ( $this->path . $subdir, $this->dir_mode );
}

$key = $subdir . "/" . $key;
}

return $key;
}

function __mkdir_recursive($pathname, $mode) {
is_dir ( dirname ( $pathname ) ) || $this->__mkdir_recursive ( dirname ( $pathname ), $mode );

return is_dir ( $pathname ) || @mkdir ( $pathname, $mode );
}

function __do_mkdir($pathname, $mode) {
if ($this->surephp5)
@mkdir ( $pathname, $mode, true );
else
$this->__mkdir_recursive ( $pathname, $mode );
}

function __scan_dir($dir, $expire_timespan = 0) {
$n = 0;
$dirstack = array ();
array_push ( $dirstack, $dir );
do {
$dir = array_pop ( $dirstack );
if (! in_array ( substr ( $dir, - 1 ), array (
"\\",
"/" 
) )) {
$dir .= "/";
}

$fs = @scandir ( $dir );
foreach ( $fs as $f ) {
if (in_array ( $f, array (
".",
".." 
) )) {
continue;
}

$fn = $dir . $f;
if (! is_readable ( $fn )) {
continue;
}

if (is_file ( $fn )) {
if ($expire_timespan > 0) {
$ts = time () - filemtime ( $fn );
#if ($ts < $expire_timespan)
if ($ts < ($expire_timespan *60*60*24))



{
continue;
}
}

if ($expire_timespan >= 0) {
@unlink ( $fn );
}

$n ++;
}
elseif (is_dir ( $fn )) {
array_push ( $dirstack, $fn );
}
}
if ($expire_timespan == 0) {
@rmdir ( $dir );
}
} while ( sizeof ( $dirstack ) > 0 );

return $n;
}
}
class WidgetCache {
var $plugin_name = 'WP Widget Cache';
#var $plugin_version = '0.26';
var $wcache;
var $cachedir;
var $wgcOptions;
var $wgcTriggers;
#var $wgcSettings;
var $wgcVaryParams;
var $wgcEnabled = true;
var $wgcAutoExpireEnabled = true;
var $wgcVaryParamsEnabled = true;
var $triggerActions = array ();
var $varyParams = array ();

function WidgetCache() {
$this->cachedir = WP_CONTENT_DIR . '/widget-cache';

$url_info = parse_url ( site_url () );

$shost = $this->array_element ( $url_info, 'host' );
$spath = $this->array_element ( $url_info, 'path' );

//maybe got many blogs under the same source
$this->cachedir .= '/' . $shost . ($spath ? '_' . md5 ( $spath ) : '');

$this->wcache = new WCache ( $this->cachedir );

if (! (is_dir ( $this->cachedir ) && is_writable ( $this->cachedir ))) {
add_action ( 'admin_notices', array (
&$this,
'widget_cache_warning' 
) );
}
else {
$this->__wgc_load_opts ();

if ($this->wgcEnabled) {
if ($this->wgcAutoExpireEnabled)
{
$this->triggerActions = array (
"category" => array (
"add_category",
"create_category",
"edit_category",
"delete_category" 
),
"comment" => array (
"comment_post",
"edit_comment",
"delete_comment",
"pingback_post",
"trackback_post",
"wp_set_comment_status" 
),
"link" => array (
"add_link",
"edit_link",
"delete_link" 
),
"post" => array (
"delete_post",
"save_post" 
),
"tag" => array (
"create_term",
"edit_term",
"delete_term" 
) 
);
}
if ($this->wgcVaryParamsEnabled) {
$this->varyParams = array (
"Home/Front Page" => array ( &$this, 'get_current_front' ),
"Category Archives" => array ( &$this, 'get_current_category' ),
"Tag Archives" => array ( &$this, 'get_current_tag' ),
"Author Archives" => array ( &$this, 'get_current_author' ),
"Search Archives" => array ( &$this, 'get_current_search' ),
"Date Archives" => array ( &$this, 'get_current_date' ),
"Single Posts" => array ( &$this, 'get_current_single' ),
"Static Pages" => array ( &$this, 'get_current_page' )

#"Page Types" => array ( &$this, 'get_current_pagesss' )
#"User Levels" => array ( &$this, 'get_user_level' ),
#"User Agents" => array ( &$this, 'get_user_agent' )
);
}
add_action ( 'wp_head', array (&$this, 'widget_cache_redirect_callback' ), 99999 );
}

if (is_admin ()) {
add_action ( 'dashmenu', array (&$this, 'dashboard_delete_wg_cache' ) );

if ($this->wgcEnabled) {
add_action ( 'sidebar_admin_page', 
array (
&$this,
'widget_cache_options_filter' 
) );
}

add_action ( 'sidebar_admin_setup', 
array (
&$this,
'widget_cache_expand_control' 
) );

if (isset ( $_GET ["wgdel"] )) {
add_action ( 'admin_notices', array (
&$this,
'widget_wgdel_notice' 
) );
}
}
}
}

function dashboard_delete_wg_cache() {
if (function_exists ( 'is_site_admin' ) && ! is_site_admin ())
return false;
if (function_exists ( 'current_user_can' ) && ! current_user_can ( 'manage_options' ))
return false;
echo "<li><a href='" . wp_nonce_url ( 'options-general.php?page=widget-cache.php&clear=1', 'widget-cache' ) .
 "' title='Clear widget cache'>Clear widget cache</a></li>";
}

function wgc_get_option($key, $default = array ()) {
$ops = get_option ( $key );

if (! $ops) {
$ops = $default;
}
else {
foreach ( $default as $k => $v ) {
if (! isset ( $ops [$k] ))
$ops [$k] = $v;
}
}

return $ops;
}

function array_element($array, $ele) {
return isset ( $array [$ele] ) ? $array [$ele] : false;
}

function __wgc_load_opts() {
$this->wgcOptions = $this->wgc_get_option ( 'widget_cache' );
$this->wgcTriggers = $this->wgc_get_option ( 'widget_cache_action_trigger' );
$this->wgcVaryParams = $this->wgc_get_option ( 'widget_cache_vary_param' );
}

function wgc_update_option($key, $value) {
update_option ( $key, $value );
}

function wp_load_default_settings() {
$wgc_ops = array ();
$this->wgc_update_option ( 'widget_cache_settings', $wgc_ops );
return $wgc_ops;
}

function widget_cache_warning() {
$pdir = WP_CONTENT_DIR . '/';

if (is_dir ( $this->cachedir )) {
$wmsg = "'$this->cachedir' is not writable, please check '$this->cachedir' permissions, and give your web server the permission to create directory and files.";
} else {
$wmsg = "'$this->cachedir' can't be created, please check '$pdir' permissions, and give your web server the permission to create directory and files.";
}

echo "<div id='widget-cache-warning' class='updated fade'><p><strong>WP Widget Cache not work!</strong><br/>$wmsg</p></div>";
}

function widget_wgdel_notice() {
$id = $_GET ["wgdel"];
$this->wcache->remove_group ( $id );
echo '<div id="widget-cache-notice" class="updated fade"><p>Delete widget cache: ' . $id . '</p></div>';
}

function widget_cache_options_filter() {
$wl_options = $this->wgcOptions;
?>
<div class="wrap">
<form method="POST">
<a name="wgcoptions"></a>
<h2><?php  echo $this->plugin_name; ?> Options</h2>
<p style="line-height: 30px;">
<span class="submit"> <input type="submit" name="widget_cache-clear"
class="button" id="widget_cache-options-submit"
value="Clear all widgets cache(<?php echo $this->wcache->cachecount(); ?>)" />
</span>
</p>
</form>
</div>
<?php
}

function widget_cache_expand_control() {
global $wp_registered_widgets, $wp_registered_widget_controls;

$wc_options = $this->wgcOptions;
$wc_trigers = $this->wgcTriggers;
$wc_varyparams = $this->wgcVaryParams;

if ($this->wgcEnabled) {
foreach ( $wp_registered_widgets as $id => $widget ) {
#if (! $wp_registered_widget_controls [$id])
if (!isset($wp_registered_widget_controls[$id]))
wp_register_widget_control ( $id, $widget ['name'], 
array (
&$this,
'widget_cache_empty_control' 
) );

if (! array_key_exists ( 0, $wp_registered_widget_controls [$id] ['params'] ) ||
 is_array ( $wp_registered_widget_controls [$id] ['params'] [0] ))
$wp_registered_widget_controls [$id] ['params'] [0] ['id_for_wc'] = $id;
else {
array_push ( $wp_registered_widget_controls [$id] ['params'], $id );
$wp_registered_widget_controls [$id] ['height'] += 40;
}

$wp_registered_widget_controls [$id] ['callback_wc_redirect'] = $wp_registered_widget_controls [$id] ['callback'];
$wp_registered_widget_controls [$id] ['callback'] = array (
&$this,
'widget_cache_extra_control' 
);
}

if ('post' == strtolower ( $_SERVER ['REQUEST_METHOD'] )) {
if (isset ( $_POST ["widget_cache-clear"] )) {
$this->wcache->clear ();
wp_redirect ( add_query_arg ( 'message', 'wgc#wgcoptions' ) );
exit ();
}

foreach ( ( array ) $_POST ['widget-id'] as $widget_number => $widget_id ) {
if (isset ( $_POST [$widget_id . '-wgc-expire'] )) {
$wc_options [$widget_id] = intval ( $_POST [$widget_id . '-wgc-expire'] );
}
if ($this->wgcAutoExpireEnabled) {
if (isset ( $_POST [$widget_id . '-wgc-trigger'] )) {
$wc_trigers [$widget_id] = ($_POST [$widget_id . '-wgc-trigger']);
}
else {
unset ( $wc_trigers [$widget_id] );
}
}
if ($this->wgcVaryParamsEnabled) {
if (isset ( $_POST [$widget_id . '-widget_cache-varyparam'] )) {
$wc_varyparams [$widget_id] = ($_POST [$widget_id . '-widget_cache-varyparam']);
}
else {
unset ( $wc_varyparams [$widget_id] );
}
}
}

$regd_plus_new = array_merge ( array_keys ( $wp_registered_widgets ), 
array_values ( ( array ) $_POST ['widget-id'] ) );
foreach ( array_keys ( $wc_options ) as $key ) {
if (! in_array ( $key, $regd_plus_new )) {
unset ( $wc_options [$key] );
if ($this->wgcAutoExpireEnabled)
unset ( $wc_trigers [$key] );
if ($this->wgcVaryParamsEnabled)
unset ( $wc_varyparams [$key] );
}
}

$this->wgc_update_option ( 'widget_cache', $wc_options );

if ($this->wgcAutoExpireEnabled)
$this->wgc_update_option ( 'widget_cache_action_trigger', $wc_trigers );

if ($this->wgcVaryParamsEnabled)
$this->wgc_update_option ( 'widget_cache_vary_param', $wc_varyparams );

$this->__wgc_load_opts ();
}
}
}

function widget_cache_empty_control() {
}

function widget_cache_extra_control() {
global $wp_registered_widget_controls;
$params = func_get_args ();

$id = (is_array ( $params [0] )) ? $params [0] ['id_for_wc'] : array_pop ( $params );

$id_disp = $id;

if ($this->wgcEnabled) {
$wc_options = $this->wgcOptions;

$value = $this->array_element ( $wc_options, $id );

if (is_array ( $params [0] ) && isset ( $params [0] ['number'] )) {
$number = $params [0] ['number'];

if ($number == - 1) {
$number = "%i%";
$value = "";
}

$id_disp = $wp_registered_widget_controls [$id] ['id_base'] . '-' . $number;
}

$value = intval ( $value );
if ($value <= 0)
$value = "";

$this->output_widget_options_panel ( $id_disp, $value );
}
else {
echo '<label style="color: gray; font-style: italic; margin-bottom: 10px; line-height: 150%;">WP Widget Cache disabled</label>';
}

$callback = $wp_registered_widget_controls [$id] ['callback_wc_redirect'];
if (is_callable ( $callback )) {
call_user_func_array ( $callback, $params );
}
}

function output_widget_options_panel($id_disp, $expire_ts) {
?>
<fieldset style="border: 1px solid #2583AD; padding: 3px 0 3px 5px; margin-bottom: 10px; line-height: 150%;">
<legend><?php echo $this->plugin_name; ?></legend>
<div>Expire in <input type='text' name='<?php echo $id_disp; ?>-wgc-expire' id='<?php echo $id_disp; ?>-wgc-expire' value='<?php echo $expire_ts; ?>' size=6 style="padding: 0" />Days <br />(Left empty means no cache) <br /> <a href='widgets.php?wgdel=<?php echo urlencode($id_disp) ?>'>Delete cache of this widget</a></div>
<?php if ($this->wgcAutoExpireEnabled): ?>
<div style="margin-top: 5px; border-top: 1px solid #ccc; clear: both;">
<div>Auto expire when these change:</div>
<?php foreach ( $this->triggerActions as $tkey => $actArr ): ?>
<?php
$checked = "";
if ($this->array_element ( $this->wgcTriggers, $id_disp ) &&
in_array ( $tkey, $this->wgcTriggers [$id_disp] ))
{
$checked = "checked=\"checked\"";
} ?>
<div style="float: left; display: inline; margin: 2px 2px 1px 2px;" nowrap="nowrap">
<input type="checkbox" <?php echo $checked;?> id="<?php echo $id_disp; ?>-wgc-trigger-<?php echo $tkey;?>" name="<?php echo $id_disp; ?>-wgc-trigger[]" value="<?php echo $tkey;?>" /> <label for="<?php echo $id_disp; ?>-wgc-trigger-<?php echo $tkey;?>"><?php echo ucwords($tkey);?></label>
</div>
<?php endforeach; ?>
</div>
<?php  endif; ?>
<?php if ($this->wgcVaryParamsEnabled): ?>
<div style="margin-top: 5px; border-top: 1px solid #ccc; clear: both;">
<div>Unique Cache for</div>
<?php foreach ( $this->varyParams as $vparam => $vfunc ): ?>
<?php
$checked = "";
if ($this->array_element ( $this->wgcVaryParams, $id_disp ) && in_array ( $vparam, $this->wgcVaryParams [$id_disp] )) {
$checked = "checked=\"checked\"";
} ?>
<div style="margin: 2px 1px 1px 0;" nowrap="nowrap">
<input type="checkbox" <?php echo $checked;?> id="<?php echo $id_disp; ?>-widget_cache-varyparam-<?php echo $vparam;?>" name="<?php echo $id_disp; ?>-widget_cache-varyparam[]" value="<?php echo $vparam;?>" /> <label for="<?php echo $id_disp; ?>-widget_cache-varyparam-<?php echo $vparam;?>"><?php echo ucwords($vparam);?></label>
</div>
<?php endforeach; ?>
<div><a href="http://stallion-theme.co.uk/stallion-responsive-wordpress-widget-cache/" target="_blank">Important Instructions</a></div>
  </div>
<?php endif;?>
</fieldset>
<?php }

function widget_cache_redirect_callback() {
global $wp_registered_widgets;
foreach ( $wp_registered_widgets as $id => $widget ) {
array_push ( $wp_registered_widgets [$id] ['params'], $id );
$wp_registered_widgets [$id] ['callback_wc_redirect'] = $wp_registered_widgets [$id] ['callback'];
$wp_registered_widgets [$id] ['callback'] = array (&$this, 'widget_cache_redirected_callback' );
}


}
#function get_user_level() { global $current_user; get_currentuserinfo (); return $current_user->wp_user_level;}
#function get_user_agent() { return $_SERVER ['HTTP_USER_AGENT'];}

function get_current_front() {
if (is_home()) { return 'is_home()'; }
elseif (is_front_page()) { return 'is_front_page()'; }
}
function get_current_category() {
if (is_category ()) { return get_query_var ( 'cat' ); }
}
function get_current_tag() {
if (is_tag ()) { return get_query_var ( 'tag' ); }
}
function get_current_author() {
if (is_author ()) { return 'is_author()'; }
}
function get_current_search() {
if (is_search ()) { return 'is_search()'; }
}
function get_current_date() {
if (is_year() || is_month() || is_day()) { return 'is_month()'; }
}
function get_current_single() {
if (is_single() & !is_front_page()) { global $post; return $post->ID; }
}
function get_current_page() {
if (is_page() & !is_front_page()) { global $post; return $post->ID; }
}

function get_current_pagesss() {
 if (is_home()) { return 'is_home()'; }
 elseif (is_front_page()) { return 'is_front_page()'; }
 elseif (is_single() & !is_front_page()) { global $post; return $post->ID; }
 elseif (is_page() & !is_front_page()) { global $post; return $post->ID; }
 elseif (is_category ()) { return get_query_var ( 'cat' ); }
 elseif (is_tag ()) { return get_query_var ( 'tag' ); }
 elseif (is_search()) { return 'is_search()'; }
 elseif (is_author()) { return 'is_author()'; }
 elseif (is_year() || is_month() || is_day()) { return 'is_month()'; }
#elseif (is_single() & !is_front_page()) { return 'is_single()'; }
#elseif (is_page() & !is_front_page()) { return 'is_page()'; }
#elseif (is_category ()) { return 'is_category()'; }
#elseif (is_tag ()) { return 'is_tag()'; }
return false;
}

function get_widget_cache_key($id)
{
$wckey = "wgcache_" . $id;

if ($this->wgcVaryParamsEnabled && isset ( $this->wgcVaryParams [$id] ))
{
foreach ( $this->wgcVaryParams [$id] as $vparam )
{
if ($this->varyParams [$vparam])
{
if (is_callable ( $this->varyParams [$vparam] ))
{
$temv = call_user_func ( $this->varyParams [$vparam] );
if ($temv)
$wckey .= "_" . $temv;
}
}
}
}

return $wckey;
}

function widget_cache_redirected_callback()
{
global $wp_registered_widgets;

// get all the passed params
$params = func_get_args ();

// take off the widget ID
$id = array_pop ( $params );

$callback = $wp_registered_widgets [$id] ['callback_wc_redirect']; // find the real callback


if (! is_callable ( $callback ))
return;

$wc_options = $this->wgcOptions;

$expire_ts = isset ( $wc_options [$id] ) ? intval ( $wc_options [$id] ) : - 1;

if ($expire_ts > 0)
{
while ( $this->wcache->save ( $this->get_widget_cache_key ( $id ), $expire_ts, null, $id ) )
{
call_user_func_array ( $callback, $params );
}
}
else
{
call_user_func_array ( $callback, $params );
}
}

}

$widget_cache = new WidgetCache ();

function widget_cache_remove($id)
{
global $widget_cache;
$widget_cache->wcache->remove_group ( $id );
}

function widget_cache_hook_trigger()
{
global $widget_cache;
if (isset ( $widget_cache->wgcTriggers ) && $widget_cache->wgcTriggers)
{
foreach ( $widget_cache->wgcTriggers as $wgid => $wgacts )
{
foreach ( $wgacts as $wact )
{
if (isset ( $widget_cache->triggerActions [$wact] ))
{
foreach ( $widget_cache->triggerActions [$wact] as $wpaction )
{
add_action ( $wpaction, create_function ( '$actionarg', 'widget_cache_remove("' . addslashes ( $wgid ) . '");' ), 10, 1 );
}
}
}
}
}
}

if ($widget_cache->wgcAutoExpireEnabled) {
widget_cache_hook_trigger (); } ?>